"""
Example usage:

from solarwinds_apm.apm_ready import solarwinds_ready
solarwinds_ready(wait_milliseconds=10000, integer_response=True)
"""

import logging
from typing import Any

from solarwinds_apm.apm_oboe_codes import OboeReadyCode
from solarwinds_apm.extension.oboe import Context

logger = logging.getLogger(__name__)


def solarwinds_ready(
    wait_milliseconds: int=3000,
    integer_response: bool=False,
) -> Any:
    """
     Wait for SolarWinds to be ready to send traces.

     This may be useful in short lived background processes when it is important to capture
     information during the whole time the process is running. Usually SolarWinds doesn't block an
     application while it is starting up.

     :param wait_milliseconds:int default 3000, the maximum time to wait in milliseconds
     :param integer_response:bool default False to return boolean value, otherwise True to
     return integer for detailed information

     :return:
     if integer_response:int code 1 for ready; 0,2,3,4,5 for not ready
     else:bool True for ready, False not ready,

     :Example:

      if not solarwinds_ready(10000):
         Logger.info("SolarWinds not ready after 10 seconds, no metrics will be sent")
    """
    rc = Context.isReady(wait_milliseconds)
    if not isinstance(rc, int) or not rc in OboeReadyCode.code_values():
        logger.warning("Unrecognized return code:{rc}".format(rc=rc))
        return OboeReadyCode.OBOE_SERVER_RESPONSE_UNKNOWN if integer_response else False
    elif rc != OboeReadyCode.OBOE_SERVER_RESPONSE_OK[0]:
        logger.warning(OboeReadyCode.code_values()[rc])

    return rc if integer_response else rc == OboeReadyCode.OBOE_SERVER_RESPONSE_OK[0]



class OboeReadyCode:
    OBOE_SERVER_RESPONSE_UNKNOWN = (0, "Oboe server : unknown error")
    OBOE_SERVER_RESPONSE_OK = (1, "Oboe server : is ready")
    OBOE_SERVER_RESPONSE_TRY_LATER = (2, "Oboe server : not ready yet, try later")
    OBOE_SERVER_RESPONSE_LIMIT_EXCEEDED = (3, "Oboe server : limit exceeded")
    OBOE_SERVER_RESPONSE_INVALID_API_KEY = (4, "Oboe server : invalid API key")
    OBOE_SERVER_RESPONSE_CONNECT_ERROR = (5, "Oboe server : connection error")

    @classmethod
    def code_values(cls):
        code_pairs = [v for k, v in cls.__dict__.items() if not k.startswith("__")]
        return {p[0]: p[1] for p in code_pairs if isinstance(p, tuple)}


class OboeReporterCode:
    """Return values of Oboe Reporter"""
    OBOE_INIT_ALREADY_INIT = -1
    OBOE_INIT_OK = 0
    OBOE_INIT_WRONG_VERSION = 1
    OBOE_INIT_INVALID_PROTOCOL = 2
    OBOE_INIT_NULL_REPORTER = 3
    OBOE_INIT_DESC_ALLOC = 4
    OBOE_INIT_FILE_OPEN_LOG = 5
    OBOE_INIT_UDP_NO_SUPPORT = 6
    OBOE_INIT_UDP_OPEN = 7
    OBOE_INIT_SSL_CONFIG_AUTH = 8
    OBOE_INIT_SSL_LOAD_CERT = 9
    OBOE_INIT_SSL_REPORTER_CREATE = 10
    OBOE_INIT_SSL_MISSING_KEY = 11

    @classmethod
    def get_text_code(cls, num):
        """Returns the textual representation of the numerical status code."""
        for k, v in cls.__dict__.items():
            if v == num:
                return k
        return None
